package com.gree.mall.manager.controller.settle.repair;

import cn.hutool.core.lang.TypeReference;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.gree.mall.manager.annotation.ZfireList;
import com.gree.mall.manager.bean.settle.repair.DailyImportSummaryVO;
import com.gree.mall.manager.constant.Constant;
import com.gree.mall.manager.exception.RemoteServiceException;
import com.gree.mall.manager.helper.ResponseHelper;
import com.gree.mall.manager.logic.settle.repair.DailyImportSummaryLogic;
import com.gree.mall.manager.utils.excel.ExcelUtils;
import com.gree.mall.manager.zfire.bean.ZfireParamBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.integration.redis.util.RedisLockRegistry;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;

@Slf4j
@RestController
@Api(value = "结算API", tags = {"结算API"})
@RequestMapping(value = "/daily/summary", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class DailyImportSummaryController {

    private final DailyImportSummaryLogic dailyImportSummaryLogic;
    private final RedisLockRegistry redisLockRegistry;

    @ZfireList
    @PostMapping("/list")
    @ApiOperation(value = "列表")
    public ResponseHelper<IPage<DailyImportSummaryVO>> page(
            @RequestBody ZfireParamBean zfireParamBean
    ) {
        IPage<DailyImportSummaryVO> page = dailyImportSummaryLogic.page(zfireParamBean);
        return ResponseHelper.success(page, new TypeReference<DailyImportSummaryVO>() {
        });
    }

    @ApiOperation("结算/扣回工单-导入")
    @PostMapping("/order/import")
    public ResponseHelper importOrderData(
            @ApiParam(required = true, value = "附件") @RequestPart("file") MultipartFile file,
            @ApiParam(required = true, value = "类型 1. repair.结算工单 2. reduce.需扣回工单") @RequestParam String type,
            @ApiParam(required = false, value = "导入批次号(覆盖导入时传入)") @RequestParam(required = false) String importBatchNo,
            HttpServletRequest request
    ) throws Exception {
        List<Object> datas = ExcelUtils.importExcel(file);
        String s = dailyImportSummaryLogic.importOrderData(type, importBatchNo, datas, request);
        return ResponseHelper.success(s);
    }

    @PostMapping("/repair/delete")
    @ApiOperation(value = "结算数据导入-删除")
    public ResponseHelper repairDelete(
            @ApiParam(required = true, value = "导入批次号") @RequestParam String importBatchNo
    ){
        dailyImportSummaryLogic.repairDelete(importBatchNo);
        return ResponseHelper.success();
    }

    @PostMapping("/repair/summary")
    @ApiOperation(value = "结算数据导入-汇总/批量汇总")
    public ResponseHelper repairSummary(
            @ApiParam(value = "导入批次号", required = true) @RequestParam List<String> importBatchNos,
            @ApiParam(value = "是否为安装费用") @RequestParam(required = false) Boolean install
    ) throws InterruptedException {
        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.LOCK_DAILY_SUMMARY);
        if(!obtain.tryLock(10, TimeUnit.SECONDS)){
            throw new RemoteServiceException("正在汇总中，请稍后再试");
        }
        try{
            for(String importBatchNo:importBatchNos) {
                try {
                    dailyImportSummaryLogic.repairSummary(importBatchNo,install);
                }catch(Exception e){
                    log.error("结算数据汇总失败",e);
                    return ResponseHelper.error("批次号:"+importBatchNo+"汇总失败");
                }
            }
        }finally {
            obtain.unlock();
        }
        return ResponseHelper.success();
    }
}
